@@ -18,6 +18,9 @@ namespace Tomboy
 		Gtk.Widget syncAddinPrefsWidget;
 		Gtk.Button resetSyncAddinButton;
 		Gtk.Button saveSyncAddinButton;
+		Gtk.CheckButton autosyncCheck;
+		Gtk.SpinButton autosyncSpinner;
+		Gtk.ComboBox rename_behavior_combo;
 		readonly AddinManager addin_manager;
 		
 		Gtk.Button font_button;
@@ -120,6 +123,32 @@ namespace Tomboy
 
 			AddActionWidget (button, Gtk.ResponseType.Close);
 			DefaultResponse = Gtk.ResponseType.Close;
+
+			Preferences.SettingChanged += HandlePreferencesSettingChanged;
+		}
+
+		void HandlePreferencesSettingChanged (object sender, NotifyEventArgs args)
+		{
+			if (args.Key == Preferences.NOTE_RENAME_BEHAVIOR) {
+				int rename_behavior = (int) args.Value;
+				if (rename_behavior < 0 || rename_behavior > 2) {
+					rename_behavior = 0;
+					Preferences.Set (Preferences.NOTE_RENAME_BEHAVIOR, rename_behavior);
+				}
+				if (rename_behavior_combo.Active != rename_behavior)
+					rename_behavior_combo.Active = rename_behavior;
+			} else if (args.Key == Preferences.SYNC_AUTOSYNC_TIMEOUT) {
+				int timeout = (int) args.Value;
+				if (timeout <= 0 && autosyncCheck.Active)
+					autosyncCheck.Active = false;
+				else if (timeout > 0) {
+					timeout = (timeout >= 5 && timeout < 1000) ? timeout : 5;
+					if (!autosyncCheck.Active)
+						autosyncCheck.Active = true;
+					if ((int) autosyncSpinner.Value != timeout)
+						autosyncSpinner.Value = timeout;
+				}
+			}
 		}
 		
 		// Page 1
@@ -184,24 +213,43 @@ namespace Tomboy
 			SetupPropertyEditor (bullet_peditor);
 
 			// Custom font...
-
+			Gtk.HBox font_box = new Gtk.HBox (false, 0);
 			check = MakeCheckButton (Catalog.GetString ("Use custom _font"));
-			options_list.PackStart (check, false, false, 0);
+			font_box.PackStart (check);
 
 			font_peditor =
 			        Services.Factory.CreatePropertyEditorToggleButton (Preferences.ENABLE_CUSTOM_FONT,
 			                                        check);
 			SetupPropertyEditor (font_peditor);
 
-			align = new Gtk.Alignment (0.5f, 0.5f, 0.4f, 1.0f);
-			align.Show ();
-			options_list.PackStart (align, false, false, 0);
-
 			font_button = MakeFontButton ();
 			font_button.Sensitive = check.Active;
-			align.Add (font_button);
+			font_box.PackStart (font_button);
+			font_box.ShowAll ();
+			options_list.PackStart (font_box, false, false, 0);
 
 			font_peditor.AddGuard (font_button);
+
+			// Note renaming bahvior
+			Gtk.HBox rename_behavior_box = new Gtk.HBox (false, 0);
+			label = MakeLabel (Catalog.GetString ("When renaming a linked note: "));
+			rename_behavior_box.PackStart (label);
+			rename_behavior_combo = new Gtk.ComboBox (new string [] {
+				Catalog.GetString ("Ask me what to do"),
+				Catalog.GetString ("Never rename links"),
+				Catalog.GetString ("Always rename links")});
+			int rename_behavior = (int) Preferences.Get (Preferences.NOTE_RENAME_BEHAVIOR);
+			if (rename_behavior < 0 || rename_behavior > 2) {
+				rename_behavior = 0;
+				Preferences.Set (Preferences.NOTE_RENAME_BEHAVIOR, rename_behavior);
+			}
+			rename_behavior_combo.Active = rename_behavior;
+			rename_behavior_combo.Changed += (o, e) =>
+				Preferences.Set (Preferences.NOTE_RENAME_BEHAVIOR,
+				                 rename_behavior_combo.Active);
+			rename_behavior_box.PackStart (rename_behavior_combo);
+			rename_behavior_box.ShowAll ();
+			options_list.PackStart (rename_behavior_box, false, false, 0);
 			
 			// New Note Template
 			// Translators: This is 'New Note' Template, not New 'Note Template'
@@ -450,6 +498,40 @@ namespace Tomboy
 			syncAddinPrefsContainer.Show ();
 			vbox.PackStart (syncAddinPrefsContainer, true, true, 10);
 
+			// Autosync preference
+			int timeout = (int) Preferences.Get (Preferences.SYNC_AUTOSYNC_TIMEOUT);
+			if (timeout > 0 && timeout < 5) {
+				timeout = 5;
+				Preferences.Set (Preferences.SYNC_AUTOSYNC_TIMEOUT, 5);
+			}
+			Gtk.HBox autosyncBox = new Gtk.HBox (false, 5);
+			// Translators: This is and the next string go together.
+			// Together they look like "Automatically Sync in Background Every [_] Minutes",
+			// where "[_]" is a GtkSpinButton.
+			autosyncCheck =
+				new Gtk.CheckButton (Catalog.GetString ("Automaticall_y Sync in Background Every"));
+			autosyncSpinner = new Gtk.SpinButton (5, 1000, 1);
+			autosyncSpinner.Value = timeout >= 5 ? timeout : 10;
+			Gtk.Label autosyncExtraText =
+				// Translators: See above comment for details on
+				// this string.
+				new Gtk.Label (Catalog.GetString ("Minutes"));
+			autosyncCheck.Active = autosyncSpinner.Sensitive = timeout >= 5;
+			EventHandler updateTimeoutPref = (o, e) => {
+				Preferences.Set (Preferences.SYNC_AUTOSYNC_TIMEOUT,
+				                 autosyncCheck.Active ? (int) autosyncSpinner.Value : -1);
+			};
+			autosyncCheck.Toggled += (o, e) => {
+				autosyncSpinner.Sensitive = autosyncCheck.Active;
+				updateTimeoutPref (o, e);
+			};
+			autosyncSpinner.ValueChanged += updateTimeoutPref;
+
+			autosyncBox.PackStart (autosyncCheck);
+			autosyncBox.PackStart (autosyncSpinner);
+			autosyncBox.PackStart (autosyncExtraText);
+			vbox.PackStart (autosyncBox, false, true, 0);
+
 			Gtk.HButtonBox bbox = new Gtk.HButtonBox ();
 			bbox.Spacing = 4;
 			bbox.LayoutStyle = Gtk.ButtonBoxStyle.End;
